with(LinearAlgebra):
with(student):

read("rk.ml"):      ## runge-kutta method from taka_runge_kutta.rr

test:=proc({grad:=0})
  option grad;
  
  read("pfdata1.ml"): ## run mk_pfdata(1) in pfaffian.rr on asir
  fb1_test1([P2,Q2,P3,Q3,R3],Lp,Ls,_options['grad']);
end proc:

fb1_test1:=proc(Ps,Lp,Ls,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  option grad;
  
  F0, T := fb_integrand(1);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;

  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
#  Ls := [dx11-dy1^2, dx12-dy1*dy2, dx22-dy2^2,
#    dx11+dx22-r^2,
#    x12*dx11+2*(x22-x11)*dx12-x12*dx22+y2*dy1-y1*dy2,
#    r*dr-2*(x11*dx11+x12*dx12+x22*dx22)-(y1*dy1+y2*dy2)-1];
#  Ls := nd_weyl_gr(Ls,[r,x11,x12,x22,y1,y2,dr,dx11,dx12,dx22,dy1,dy2],0,0);
  SV := [s11,s12,s22,s1,s2];
  #  S = [1,0.5,-1.2,-0.3,-0.13]; # ¬
  S := [0.012,0.313,-0.043,0.501,-0.035];
  G := -(s11*x11+s12*x12+s22*x22+s1*y1+s2*y2); # exp(G) * integral_part
  Xmin := [-3,-3,0.1,0.1,0.1];
  Xmax := [1,10,3,3,3];
  Pos := [-3, 0.5, 0.15, 1, 0.15];
  Bound := [0.0001,1000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:


astro:=proc({grad:=0})
  option grad;
  
  read("pfdata2.ml"): ## run mk_pfdata(2) in pfaffian.rr on asir
  fb1_astro([P2,Q2,P3,Q3,R3],Lp,Ls,_options['grad']);
end proc:

fb1_astro:=proc(Ps,Lp,Ls,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  option grad;
  
  F0, T := fb_integrand(2);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;

  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := [s11,s12,s13,s22,s23,s33,s1,s2,s3];
  #  S = [1,0.5,-1.2,-0.3,-0.13]; # ¬
  S := [0.3119,  0.0292,  0.0707,
                 0.3605,  0.0462,
                          0.3276,
       -0.0063, -0.0054, -0.0762];
  G := -(s11*x11+s12*x12+s13*x13+s22*x22+s23*x23+s33*x33+s1*y1+s2*y2+s3*y3); # exp(G) * integral_part
  Xmin := [-30,-30,-30,-30, -30,   -30,   -30,    -30, -30];
  Xmax :=  [10, 10, 10, 10,  20, -0.01, -0.01, -0.001,  10];
  Pos := [-0.161,  0.4377,  1.0604,
                   0.2038,  0.6924,
                           -0.0428,
          -0.019, -0.0162, -0.2286];
  Bound := [0.0001,1000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:


mag:=proc({grad:=0})
  option grad;
  
  read("pfdata2.ml"): ## run mk_pfdata(2) in pfaffian.rr on asir
  fb1_mag([P2,Q2,P3,Q3,R3],Lp,Ls,_options['grad']);
end proc:

fb1_mag:=proc(Ps,Lp,Ls,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  option grad;
  
  F0, T := fb_integrand(2);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;
  
  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := [s11,s12,s13,s22,s23,s33,s1,s2,s3];
  #  S = [1,0.5,-1.2,-0.3,-0.13]; # ¬
  S := [0.045, -0.075,  0.014,
                0.921, -0.122,
                        0.034,
        0.082, -0.959,  0.131];
  G := -(s11*x11+s12*x12+s13*x13+s22*x22+s23*x23+s33*x33+s1*y1+s2*y2+s3*y3); # exp(G) * integral_part
  Xmin := [-30,-30,-30,-30, -30,   -30, -30,    -32, -30];
  Xmax :=  [30, 30, 30, 30,  30, -0.01,  30, -0.001,  32];
  Pos := [6.695, 0.048,  3.502,
                 6.459, 22.902,
                       -13.693,
          1.712,-28.99, 29.992];
  Bound := [0.0001,1000];
  Dh := 0.01;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:

test3:=proc({grad:=0})
  option grad;
  
  read("pfdata3.ml"): ## run mk_pfdata(3) in pfaffian.rr on asir
  fb1_test3([P2,Q2,P3,Q3,R3],Lp,Ls,_options['grad']);
end proc:

fb1_test3:=proc(Ps,Lp,Ls,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  option grad;
  
  F0, T := fb_integrand(3);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;
  
  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := [s11,s12,s13,s14,s22,s23,s24,s33,s34,s44,s1,s2,s3,s4];
  #  S = [1,0.5,-1.2,-0.3,-0.13]; # ¬
  S := [0.045, -0.075,  0.014, 0.045,
                0.921, -0.122, 0.298,
                        0.034, 0.021,
                               0.087,
        0.082, -0.959,  0.131, 0.064];
  G := -(s11*x11+s12*x12+s13*x13+s14*x14+s22*x22+s23*x23+s24*x24+s33*x33+s34*x34+s44*x44+s1*y1+s2*y2+s3*y3+s4*y4); # exp(G) * integral_part
  Xmin := [-2, -2, -2, -2, -2, -2, -2, -4, -2, -2, -2, -2, -2, -2];
  Xmax := [ 2,  2,  2,  2,  2,  2,  2, -2,  2, -1,  2,  2,  2,  2];
  Pos := [-1.995, -1.722,  1.362,  1.123,
                   1.999, -1.998,  1.995,
                          -2.033,  1.539,
                                  -0.342,
           0.972,  -0.12, -1.992,  0.518];
  Bound := [0.0001,1000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:

testn:=proc({grad:=0})
  local Name, TL, SX, SY, N;
  option grad;
  
  Name := "tmp-fb.txt";
  TL := read_point_data(Name);
  SX, SY := mk_s(TL);
  N := op(1,SY)-1;
print(N);
  read(cat("pfdata",N,".ml")); ## run mk_pfdata() in pfaffian.rr on asir
  fb1_testn([P2,Q2,P3,Q3,R3],Lp,Ls,SX,SY,_options['grad']);
end proc:

fb1_testn:=proc(Ps,Lp,Ls,SX,SY,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  option grad;

  F0, T := fb_integrand(Ls[1]);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;
  
  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := map(x2s,V);
  # ¬
  S := slist(SX,SY);
  #G := -(s11*x11+s12*x12+s13*x13+s14*x14+s22*x22+s23*x23+s24*x24+s33*x33+s34*x34+s44*x44+s1*y1+s2*y2+s3*y3+s4*y4); # exp(G) * integral_part
  G := -Vector[row](V).Vector(S);
  Xmin := [0.5,0,0,3.5,0,5.5,0.5,1.5,2.5];
  Xmax := [1.5,1,1,4.5,1,6.5,1.5,2.5,3.5];
  Pos := [1,0,0.2,4,0,6,1,2,3];
  Bound := [0.0001,100000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:

testn1:=proc({grad:=0})
  local Name, TL, SX, SY, N, R, AIC;
  option grad;
  
  Name := "tmp-fb-n1.txt";
  TL := read_point_data(Name);
  SX, SY := mk_s(TL);
  N := op(1,SY)-1;
print(N);
  read(cat("pfdata",N,".ml")); ## run mk_pfdata() in pfaffian.rr on asir
  R := fb1_testn1([P2,Q2,P3,Q3,R3],Lp,Ls,SX,SY,_options['grad']);
  AIC := fb_aic_val(R[1][1],nops(TL),N+1);
  printf("AIC value: %a\n",AIC);
  return(R);
end proc:

fb1_testn1:=proc(Ps,Lp,Ls,SX,SY,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  option grad;

  F0, T := fb_integrand(Ls[1]);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;
  
  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := map(x2s,V);
  # ¬
  S := slist(SX,SY);
  #G := -(s11*x11+s12*x12+s13*x13+s14*x14+s22*x22+s23*x23+s24*x24+s33*x33+s34*x34+s44*x44+s1*y1+s2*y2+s3*y3+s4*y4); # exp(G) * integral_part
  G := -Vector[row](V).Vector(S);
  Xmin := [-0.5$5];
  Xmax := [2,9,4,2,1];
  Pos := [0.8765432,0.276541,2.887659,1.211186,0.2098759];
  Pos := [1.5,1/10+0.4,3-0.5,1+0.3,0-0.2];
  Pos := [1,8,3,1,0];
  Bound := [0.0001,100000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:

testn3:=proc({grad:=0})
  local Name, TL, SX, SY, N, R, AIC;
  option grad;
  
  Name := "tmp-fb-n3.txt";
  TL := read_point_data(Name);
  SX, SY := mk_s(TL);
  N := op(1,SY)-1;
print(N);
  read(cat("pfdata",N,".ml")); ## run mk_pfdata() in pfaffian.rr on asir
  R := fb1_testn3([P2,Q2,P3,Q3,R3],Lp,Ls,SX,SY,_options['grad']);
  AIC := fb_aic_val(R[1][1],nops(TL),N+1);
  printf("AIC value: %a\n",AIC);
  return(R);
end proc:

fb1_testn3:=proc(Ps,Lp,Ls,SX,SY,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  option grad;

  F0, T := fb_integrand(Ls[1]);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;
  
  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := map(x2s,V);
  # ¬
  S := slist(SX,SY);
  G := -Vector[row](V).Vector(S);
  Xmin:= [0.5,4.5,-1.5,2,  1.0,-0.2,-6,  -1.5,-4.5, -1.0,  -0.5,1.7,0.5,-1.5];
  Xmax:= [1.5,  5,-1  ,3,  1.5,0.2 ,-5,  -0.5,-3.5, -0.5,   0.5,2.3,1.5,-0.5];
  Pos := [1,2*2.4+0.001,-2*0.7+0.003,2*1.3-0.002,
          1.3+0.003,0,-2*2.6+0.001,
          -1+0.0011,-2*2+0.0017,
          -1+0.004,
          0+0.00031, 2-0.00011,1+0.00031,-1];
  Bound := [0.0001,100000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:

testn4:=proc({grad:=0})
  local Name, TL, SX, SY, N, R, AIC;
  option grad;
  
  Name := "tmp-fb-n4.txt";
  TL := read_point_data(Name);
  SX, SY := mk_s(TL);
  N := op(1,SY)-1;
print(N);
  read(cat("pfdata",N,".ml")); ## run mk_pfdata() in pfaffian.rr on asir
  R:=fb1_testn4([P2,Q2,P3,Q3,R3],Lp,Ls,SX,SY,_options['grad']);
  AIC := fb_aic_val(R[1][1],nops(TL),N+1);
  printf("AIC value: %a\n",AIC);
  return(R);
end proc:

fb1_testn4:=proc(Ps,Lp,Ls,SX,SY,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R, M;
  option grad;

  F0, T := fb_integrand(Ls[1]);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;
  
  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := map(x2s,V);
  # ¬
  S := slist(SX,SY);
  G := -Vector[row](V).Vector(S);
  Pos := [1+0.001, 0,2+0.003,-1-0.0013,-2+0.0012,
           -1+0.003,2+0.007,-1+0.005, 3+0.00011,
              2-0.007,-1+0.0013, 2+0.00014,
                -2-0.00097, 1+0.00071,
                    3+0.00011,
    1/2,3/2,5/2,-3/2,-5/2];
  M :=[0.5$((Ls[1]+1)*(Ls[1]+4)/2)];
  Xmin := Pos-M;
  Xmax := Pos+M;
  Bound := [0.0001,100000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:


tests:=proc(Name,N,{grad:=0})
  local TL, Pos, S, R, AIC;
  option grad;

  TL := read_point_data(Name);
  Pos := TL[1];
  S := TL[2];
print(N);
  read(cat("pfdata",N,".ml")); ## run mk_pfdata() in pfaffian.rr on asir
  R:=fb1_tests([P2,Q2,P3,Q3,R3],Lp,Ls,Pos,S,_options['grad']);
#  AIC := fb_aic_val(R[1][1],nops(TL),N+1);
#  printf("AIC value: %a\n",AIC);
  return(R);
end proc:

fb1_tests:=proc(Ps,Lp,Ls,Pos,S,{grad:=0})
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, G;
  local Xmin, Xmax, Bound, Dh, R, M;
  option grad;

  F0, T := fb_integrand(Ls[1]);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;

  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
  SV := map(x2s,V);
  G := -Vector[row](V).Vector(S);
  M :=[1$((Ls[1]+1)*(Ls[1]+4)/2)];
  Xmin := Pos-M;
  Xmax := Pos+M;
  Bound := [0.0001,100000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,_options['grad']); # check=1
  return(R);
end proc:


fb1:=proc(F,CPs,CLp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,CPos,Bound,Dh,{grad:=0})
  local AV, DAV, XYRule, PF, Base, Val, SRule, GS, R, Ps, Lp, Grad;
  local Pos:=CPos;
  local P, DN, INT;
  option grad;

  Ps := CPs;
  Lp := CLp;

#  if ( type(Pos=getopt(pos)) == -1 ) Pos = vtol((ltov(Xmin)+ltov(Xmax))*0.5);
#  if ( type(Bound=getopt(bound)) == -1 ) Bound = 0;
#  if ( type(Check_int=getopt(check)) == -1 ) Check_int = 0;

#  bsave([F,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos],"fb1_input.ab");

#  S := map(decimal2rat,S);
  AV := [op(V),op(RV)];
  DAV := [op(DV),op(map(fb1_add_d,RV))];
  Pos := [op(Pos),op(RVV)];
  # /* e.g. tk_pfn.consRule([x,y,z],[1,2,3]) => [[x,1],[y,2],[z,3]] */
  XYRule := subsrule(AV,Pos);
  #/* holonomic ideal Ls  Pfaffian ľ, RV([r])  RVV([1])  */
#  PF := fb1_mypf_rest(Ls,AV,DAV,RV,RVV);
  Ps := subs(subsrule(RV,RVV),Ps);
  Lp := subs(subsrule(RV,RVV),Lp);

  Base := subs(subsrule(DAV,AV),Ls[9]);
  if ( Ls[1] < 3 ) then
    Val := fb1_intval(F,AB,Base,AV,XYRule,X);
  else
    Val := fb_int_tkoyama(Ls[1],1,CPos);
  end if;
  SRule := subsrule(SV,S);
  GS := subs(SRule,G);

  # /* GS : exp(G)|_¬ */
  R := fb1_alldir(Ps,Lp,Ls,V,RV,GS,Val,Pos,Xmin,Xmax,Bound,Dh,_options['grad']); # | bound = Bound);
#  if ( type(R[0]) != 7 && Check_int ) {
#    print("");
    printf("check intval:\n");
    P := [op(convert(R[2],`list`)),op(RVV)];
    AV := [op(V),op(RV)];
    DN := eval(subs(subsrule([op(AV),op(SV)],[op(P),op(S)]),exp(-GS)));
    XYRule := subsrule(AV,P);
#    STM := bload("fb1_pfn_all.ab");
    if ( Ls[1] < 3 ) then
      INT := fb1_intval(F,AB,Base,AV,XYRule,X);
    else
      INT := fb_int_tkoyama(Ls[1],1,convert(R[2],`list`));
    end if;
    printf("%a\n",convert((Vector(INT)/DN),`list`));
#  }
  return(R);
end proc:

fb_aic_val:=proc(L,N,Dim)
  local S, NV;

  S := evalf((2*Pi^(Dim/2))/GAMMA(Dim/2));
  printf("surface area: %a\n",S);
  printf("log: %a\n",log(L/S));
  NV := Dim*(Dim+3)/2;
  return(2*N*log(L/S) + 2*(NV-1));
end proc:

fb1_add_d:=proc(X)
  return(cat(d,convert(X,string)));
end proc:

fb1_intval:=proc(F,CAB,STM,V,Rule,CX)
  local i, V0, DF, FS, IV, X, AB;
  X := CX;
  AB := CAB;
  V0 := [];
  if ( nops(X) = 1 ) then   ## int ٤Τ1ʬ Doubleint 
    X := [op(X),tmp];
    AB := [op(AB),[0,1]];
  end if;
  for i from 1 to op(1,STM) do
    DF := fb1_diff(F,STM[i],V);
    FS := subs(Rule,DF);
    if ( nops(X) = 2 ) then
      IV := evalf(Doubleint(FS,X[1]=AB[1][1]..AB[1][2],X[2]=AB[2][1]..AB[2][2]));
    else
      IV := evalf(Tripleint(FS,X[1]=AB[1][1]..AB[1][2],X[2]=AB[2][1]..AB[2][2],X[3]=AB[3][1]..AB[3][2]));
    end if;
    V0 := [op(V0),IV];
  od;
  return V0;
end proc:

fb1_diff:=proc(F,M,V)
  local i, MV, CF;
  CF := F;
  MV := map[2](degree,M,V);
  for i from 1 to nops(MV) do
    if ( MV[i] <> 0 ) then 
      CF := diff(CF,V[i]$MV[i]);
    end if;
  od;
  return(CF);
end proc:

test_fb1_intval:=proc()
  local F0, PC, F, X, AB, STM, V, Pos, Rule, R;
  
  F0 := t1^2*x11+t1*t2*x12+t1*t3*x13+t1*t4*x14+t2^2*x22+t2*t3*x23+t2*t4*x24+t3^2*x33+t3*t4*x34+t4^2*x44+y1*t1+y2*t2+y3*t3+y4*t4;
  PC := [t1 = cos(h1), t2 = sin(h1)*cos(h2), t3 = sin(h1)*sin(h2)*cos(h3), t4 = sin(h1)*sin(h2)*sin(h3)];
  F := exp(subs(PC,F0))*(sin(h1))^2*sin(h2);
  X := [h1, h2, h3];
  AB := [[0, Pi], [0, Pi], [0, 2*Pi]];
  STM := Vector([1,y1,y2,y3,y4,y1^2,y2^2,y3^2]);
  V := [x11,x12,x13,x14,x22,x23,x24,x33,x34,x44,y1,y2,y3,y4];
  Pos := [-1.995, -1.722,  1.362,  1.123,
                   1.999, -1.998,  1.995,
                          -2.033,  1.539,
                                  -0.342,
           0.972,  -0.12, -1.992,  0.518];
  Rule := subsrule(V,Pos);  ## subsrule([x,y,z],[1,2,3]) returns [x=1,y=2,z=3]
  R := fb1_intval(F,AB,STM,V,Rule,X);
  return(R);
end proc:

test_fb1_intval2:=proc()
  local F0, PC, F, X, AB, STM, V, Pos, Rule, R;
  
  F0 := t1^2*x11+t1*t2*x12+t1*t3*x13+t2^2*x22+t2*t3*x23+t3^2*x33+y1*t1+y2*t2+y3*t3;
  PC := [t1 = cos(h1), t2 = sin(h1)*cos(h2), t3 = sin(h1)*sin(h2)];
  F := exp(subs(PC,F0))*sin(h1);
  X := [h1, h2];
  AB := [[0, Pi], [0, 2*Pi]];
  STM := Vector([1,y1,y2,y3,y1^2,y2^2]);
  V := [x11,x12,x13,x22,x23,x33,y1,y2,y3];
  Pos := [1.1, 1.2, 1.3, 2.2, 2.3, 3.3, 0.1, 0.2, 0.3];
  Rule := subsrule(V,Pos);  ## subsrule([x,y,z],[1,2,3]) returns [x=1,y=2,z=3]
  R := fb1_intval(F,AB,STM,V,Rule,X);
  return(R);
end proc:

subsrule:=proc(L,V)
  local n,i,R;
  n:=nops(L);
  R:=[];
  for i from 1 to n do
    R := [op(R),L[i]=V[i]]:
  od:
end proc:

mysubs:=proc(E,L,V)
  local Rule;
  Rule:=subsrule(L,V);
  evalf(subs(Rule,E));
end proc:


fb1_alldir:=proc(Ps,Lp,Ls,V,RV,GS,Val,Pos,Xmin,Xmax,Bound,Dh,{grad:=0})
  local i;
  local TMP_VAR, AV, N, H0, Rule, G0, H, Ip, SPF, R, Grad;
  option grad;
  
  TMP_VAR := zzzz;
#  if ( type(Dh=getopt(dh)) == -1 ) Dh = 0.05;
#  if ( type(Bound=getopt(bound)) == -1 ) Bound = 0;

  AV := [op(V),op(RV)];
  N := nops(Val); #/* N : PF  row size */
  H0 := Vector(Val);
  Rule := subsrule(AV,Pos);
  G0 := evalf(subs(Rule,exp(GS)));
  H0 := convert(H0*G0,'list');
  printf("initial value of int:\n");
  print(Val);
  printf("initial value of F(x,y,r=1):\n");
  print(H0);
  # /* size N ѿꥹ H */
  H := Vector(N);
  for i from 1 to N do
    H[i] := cat(TMP_VAR,i);
  od:
  # /* base_replace_n([x,y,z], [[x,1],[y,2],[z,3]]) => [1,2,3] */
  Ip := subs(Rule,V);

#  SPF := fb1_diag_shift(PF,AV,GS);
  if ( grad > 0 ) then 
    R := fb1_hlml_grad(Ps,Lp,Ls,GS,V,H,Ip,H0,Xmin,Xmax,Dh,Bound,1);
  else
    R := fb1_hlml_fast(Ps,Lp,Ls,GS,V,H,Ip,H0,Xmin,Xmax,Dh,Bound,1); #Deviation=1);
  end if;
  return(R);
end proc:


fb1_hlml_fast:=proc(Ps,Lp,Ls,GS,V,F,Ip,Finit,Vmin,Vmax,Dh,Bound,Deviation)
#  extern HLML_path, HLML_INT_VAL;
  local i;
  local Debug, Cnt, Cp, N, Id, Id2, PPA;
  local Tf1, Tf2, Rule, A, Min, Tf, PP, Update, IND, DPs;
  
  local TRUE;
  
  Debug := 2;
#  if ( type(Deviation=getopt(deviation)) == -1 ) Deviation=0;
#  if ( type(Bound=getopt(bound)) == -1 ) Bound = 0;
#  print("hlml_fast"); # print(getopt());

  Cnt := -1;
  Cp := Vector(Ip); #/* ߰ */
  Tf := Finit;    #/* ߤ */

  IND := map(mk_index,V,Ls[2],Ls[3]);

  DPs := mkdmat(Ps,Lp,Ls);

#  HLML_path=[vtol(Cp)];
#  HLML_INT_VAL = [Tf[0]];

  N := nops(V);
  Id := -1;
  Id2 := -1;
  PPA := Vector(N);
  TRUE := true;
  while ( TRUE ) do
#    #/*  
#    #if (nearSing(Dnn,V,vtol(Cp),Dh)) {
#    #  printf("%a is too near the singularity\n",Cp);
#    #  error("Singularity");
#    #}
#    #*/
    Cnt := Cnt + 1;
    Tf1 := Vector(N);  # /* V[I] += Dh Ǥ. */
    Tf2 := Vector(N);  # /* V[I] -= Dh Ǥ. */
    Rule := subsrule(V,Cp);

    if (Id >= 0) then

      i := Id; Id := -1;
      if ( Update > 0 ) then
        if (Cp[i]+Dh <= Vmax[i]) then
          A := runge_kutta_4a_linear(PP,V[i],IND[i],F,Cp[i],Tf,Cp[i]+Dh,Dh/2,0,1);
          Tf1[i] := subsop(1=NULL,A[1]); 
        end if;
      else
        if (Cp[i]-Dh >= Vmin[i]) then
          A := runge_kutta_4a_linear(PP,V[i],IND[i],F,Cp[i],Tf,Cp[i]-Dh,Dh/2,0,1);
          Tf1[i] := subsop(1=NULL,A[1]);
        end if;
      end if;
      Min := Tf[1];
      if ( not Tf1[i] = 0) then
        if (Tf1[i][1] < Min) then
          if ( not Deviation = 0) then
            stopDeviation(Tf1[i][1],Min,Deviation);
          end if;
          Id := i;
          Tf := Tf1[i];
        end if;
      end if;
    end if;

    if (Id < 0) then
      for i from 1 to N do
        if ( Debug > 1 ) then printf("i=%a\n",i); end if;
#        PP := Pf[i];
        if ( Debug > 1 ) then print("base_replace_n"); end if;
        PPA[i] := subs(subsop(i=NULL,Rule),[Ps,Lp,DPs,GS]);
        PP := PPA[i];
        if ( Debug > 1) then print("Runge-kutta"); end if;
        if (Cp[i]+Dh <= Vmax[i]) then
          A := runge_kutta_4a_linear(PP,V[i],IND[i],F,Cp[i],Tf,Cp[i]+Dh,Dh/2,0,1);
          Tf1[i] := subsop(1=NULL,A[1]);
          #/* A=[[,V[I], F[0], F[1], ...], ....] */
        end if;
        if (Cp[i]-Dh >= Vmin[i]) then
          A := runge_kutta_4a_linear(PP,V[i],IND[i],F,Cp[i],Tf,Cp[i]-Dh,Dh/2,0,1);
          Tf2[i] := subsop(1=NULL,A[1]);
        end if;
      od;
      Min := Tf[1]; Update := 0;
      for i from 1 to N do
        if ( Tf1[i] <> 0) then
          if (Tf1[i][1] < Min) then
            if ( not Deviation > 0) then 
              stopDeviation(Tf1[i][1],Min,Deviation);
            end if;
            Id := i;
            Update := Dh;
            Min := Tf1[i][1];
            Tf := Tf1[i];
          end if;
        end if;
        if ( Tf2[i] <> 0) then
          if (Tf2[i][1] < Min) then
            if ( not Deviation > 0) then
              stopDeviation(Tf1[i][1],Min,Deviation);
            end if;
            Id := i;
            Update := -Dh;
            Min := Tf2[i][1];
            Tf := Tf2[i];
          end if;
        end if;
      od:
    end if;
    if ( Update <> 0) then
      if ( Debug = 1 ) then
        if ( Id <> Id2 ) then print(Id,2); end if;
        print(mysign(Update),2);
        Id2 := Id;
      end if;
      Cp[Id] := Cp[Id]+Update; #/*  */
      if ( Debug > 1 ) then print([Cnt,Id,Cp,Tf]); end if;
#      HLML_path=cons(vtol(Cp),HLML_path);
#      HLML_INT_VAL = cons(Tf[0],HLML_INT_VAL);
      if ( Bound <> 0) then
        if ((Min < Bound[1]) or (Min > Bound[2])) then
          printf("Exceeded the bounds, bound=%a\n",Bound); 
          return(["Out of bound",Cp,Tf]);
        end if;
      end if;
      PP := PPA[Id];
    else
      return([Cp,Tf]);
    end if;
  od;
end proc:


#### gradient method
testg:=proc()
  read("pfdata1.ml"): ## run mk_pfdata(1) in pfaffian.rr on asir
  fb1_testg1([P2,Q2,P3,Q3,R3],Lp,Ls);
end proc:

fb1_testg1:=proc(Ps,Lp,Ls)
  local F0, F1, F, AB, X, T, J, V, DV, RV, RVV, SV, S, G;
  local Xmin, Xmax, Pos, Bound, Dh, R;
  
  F0, T := fb_integrand(1);
  F1, J, X, AB := polar(T,[Ls[4],h]);
  F := exp(subs(F1,F0))*J;

  V := Ls[8][1..nops(Ls[8])/2-1]; #[x11,x12,x22,y1,y2];
  DV := Ls[8][nops(Ls[8])/2+1..nops(Ls[8])-1]; #[dx11,dx12,dx22,dy1,dy2];
  RV := [Ls[4]];
  RVV := [1];
#  Ls := [dx11-dy1^2, dx12-dy1*dy2, dx22-dy2^2,
#    dx11+dx22-r^2,
#    x12*dx11+2*(x22-x11)*dx12-x12*dx22+y2*dy1-y1*dy2,
#    r*dr-2*(x11*dx11+x12*dx12+x22*dx22)-(y1*dy1+y2*dy2)-1];
#  Ls := nd_weyl_gr(Ls,[r,x11,x12,x22,y1,y2,dr,dx11,dx12,dx22,dy1,dy2],0,0);
  SV := [s11,s12,s22,s1,s2];
  #  S = [1,0.5,-1.2,-0.3,-0.13]; # ¬
  S := [0.012,0.313,-0.043,0.501,-0.035];
  G := -(s11*x11+s12*x12+s22*x22+s1*y1+s2*y2); # exp(G) * integral_part
  Xmin := [-3,-3,0.1,0.1,0.1];
  Xmax := [1,10,3,3,3];
  Pos := [-3, 0.5, 0.15, 1, 0.15];
  Bound := [0.0001,1000];
  Dh := 0.05;
  R := fb1(F,Ps,Lp,Ls,AB,X,V,DV,RV,RVV,G,SV,S,Xmin,Xmax,Pos,Bound,Dh,grad=1); # check=1
  return(R);
end proc:

fb1_hlml_grad:=proc(Ps,Lp,Ls,GS,V,F,CIp,CFinit,Vmin,Vmax,CDh,Bound,Deviation)
  local DPs, PFs, GRAD, CGRAD, XT, Rule, Finit, Ip, L;
  local TPs, TLp, TDPs, VT, P, A, IND, Norm, NIp, Dh, Dh1;
  local FLAG, Cnt, Norm_bound;
  
  Norm_bound := evalf(CDh/100);
  
  Dh := CDh;
  VT := t;
  Finit := CFinit;
  Ip := Vector(CIp);
  Cnt := 0;

  DPs := mkdmat(Ps,Lp,Ls);
  IND := map(mk_index,V,Ls[2],Ls[3]);
  while ( true ) do
    Dh1 := Dh;
    Cnt := Cnt+1;
    PFs := pfs(Ps,Lp,Ls,DPs,V,Ip,GS,1);
    GRAD := Vector(map(pick_grad,PFs,Vector(Finit)));
    FLAG := 1;

    while ( FLAG = 1 ) do
#print("before: ",GRAD,FLAG);
      Norm := VectorNorm(GRAD,2);
      if ( Norm < Norm_bound ) then printf("norm cond: %f\n",Norm); return(Finit,Ip); end if;
      CGRAD := evalf[16](-GRAD/Norm);
      XT := Vector(Ip)+CGRAD*VT;
      NIp := subs(VT=Dh1,XT);
#print(Ip,NIp,Vmin,Vmax);
      FLAG, L := bound_cond(Ip,NIp,GRAD,CGRAD,Vmin,Vmax);
      if ( L <> infinity ) then Dh1 := L; FLAG := 1; end if;
#print("after: ",GRAD,FLAG,L);
    od;
    GRAD := CGRAD;
    printf("Norm: %f\n",Norm);
    Rule := subsrule(V,convert(XT,`list`));

    TPs := subs(Rule,Ps);
    TLp := subs(Rule,Lp);
    TDPs := subs(Rule,DPs);
    A := runge_kutta_4a_linear_grad([TPs,TLp,TDPs,GS],VT,[V,IND],F,0,Finit,0+Dh1,Dh1/2,0,1,GRAD);
    if ( Finit[1] < A[1][2] ) then
      Dh := Dh/2;
    else
      Ip := NIp;
      Finit := subsop(1=NULL,A[1]);
      print(Cnt,Finit,convert(Ip,`list`));
    end if;
  od;
  return(A);
end proc:


bound_cond:=proc(Ip,NIp,GRAD,CGRAD,Xmin,Xmax)
  local i, FLAG, Eps, C, L;

  Eps := 0.001;
  FLAG := 0;
  L := [];
  for i from 1 to nops(Xmin) do
    if ( NIp[i] < Xmin[i] ) then
      if ( Xmin[i] - Eps < Ip[i] and Ip[i] < Xmin[i] + Eps ) then 
        GRAD[i] := 0;
        FLAG := 1;
      else
      if ( CGRAD[i] < 0 ) then C := Xmin[i]; else C := Xmax[i]; end if;
      L := [op(L),(C-Ip[i])/CGRAD[i]];
      end if;
    end if;
    if ( Xmax[i] < NIp[i] ) then
      if ( Xmax[i] - Eps < Ip[i] and Ip[i] < Xmax[i] + Eps ) then 
        GRAD[i] := 0;
        FLAG := 1;
      else
      if ( CGRAD[i] < 0 ) then C := Xmin[i]; else C := Xmax[i]; end if;
      L := [op(L),(C-Ip[i])/CGRAD[i]-Eps^2];
      end if;
    end if;
  od;
  return(FLAG,min(L));
end proc:


bound_cond_old:=proc(Ip,NIp,GRAD,Xmin,Xmax)
  local i, FLAG, Eps;
#print(Ip,Xmin,Xmax);

  FLAG := 0;
  for i from 1 to nops(Xmin) do
    if ( Ip[i] < Xmin[i] ) then
      GRAD[i] := 0;
      FLAG := 1;
    end if;
    if ( Xmax[i] < Ip[i] ) then
      GRAD[i] := 0;
      FLAG := 1;
    end if;
  od;
  return(FLAG);
end proc:


bound_t:=proc(Ip,GRAD,Xmin,Xmax)
  local i, C, L;
  
  L := [];
  for i from 1 to nops(Ip) do
    if ( GRAD[i] < 0 ) then
      C := Xmin[i];
    else
      C := Xmax[i];
    end if;
    L := [op(L),(C-Ip[i])/GRAD[i]];
  od;
  return(min(L));
end proc:


pfs:=proc(CPs,CLp,Ls,CDPs,V,Ip,CGS,Sign)
  local i, j;
  local N, M;
  local DB, DC, HIJ, AI, H;
  local PFX, PFY;
  local P2I, P3I;
  local PF, SF, GS;
  local Ps, Lp, DPs, Rule;

  GS := CGS;
  if ( Sign < 0 ) then GS := subs(subsrule(V,-V),GS); end if;
  
  Rule := subsrule(V,Ip);
  Ps := subs(Rule,CPs);
  Lp := subs(Rule,CLp);
  DPs := subs(Rule,CDPs);
  
  N := nops(Lp); #Ls[1]+1; #RowDimension(Ps[3]);
  M := RowDimension(Lp[1][1]);
  SF := Matrix(M,M,shape=identity);

  DB := DPs[1];
  DC := DPs[2];
  AI := [];
  H := [];
  PFX := [];
  PFY := [];

  P2I := MatrixInverse(Ps[1]);
  P3I := MatrixInverse(Ps[3]);
  for i from 1 to N do
    AI := [op(AI), MatrixInverse(Lp[i][1])];  # A^(-1);
    H := [op(H), AI[i].(Lp[i][2]-Lp[i][3].P2I.Ps[2]+Lp[i][4].P3I.(Ps[4].P2I.Ps[2]-Ps[5]))];
    PF := AI[i].(Lp[i][2]-Lp[i][3].P2I.Ps[2]+Lp[i][4].P3I.(Ps[4].P2I.Ps[2]-Ps[5]));
    PFY := [op(PFY), Sign*(PF+subs(Rule,diff(GS,Ls[3][i]))*SF)];
  od;

  for i from 1 to N do
    for j from i to N do
      HIJ := AI[i].( DPs[1][i][j]-DPs[2][i][j].P2I.Ps[2]-Lp[i][3].P2I.DPs[3][j]+Lp[i][4].P3I.(DPs[4][j].P2I.Ps[2]+Ps[4].P2I.DPs[3][j]-DPs[5][j]) );
      PF := HIJ + H[i].H[j];
      PFX := [op(PFX), Sign*(PF+subs(Rule,diff(GS,Ls[2][i][j]))*SF)];
    od;
  od;

  return([op(PFX),op(PFY)]);
end proc:


pick_grad:=proc(PF,Val)
  return(PF[1].Val);
end proc:



vec_clone:=proc(V)
  local N, C, i;
  N := op(1,V);
  C := Vector(op(1,V));
  for i from 1 to N do
    C[i] := V[i];
  od:
  return(C);
end proc:

mysign:=proc(A)
  if ( A > 0 ) then return("+"); else if (A < 0) then print("-") end if; end if;
end proc:

mk_index:=proc(X,MX,VY)
  local i, j;
  local R, C;
  R,C := Dimensions(MX);
  for i from 1 to R do
    for j from 1 to C do
      if ( X = MX[i][j] ) then return([i,j]); end if;
    od;
  od;

  R := op(1,VY);
  for i from 1 to R do
    if ( X = VY[i] ) then return([i]); end if;
  od;
end proc:


mkdmat:=proc(Ps,Lp,Ls)
  local i,j;
  local N,X,Y;
  local DB,DC,DBI,DCI,DQ2,DQ3,DR3;
  local H;
  
  N := Ls[1];
  X := Ls[2];
  Y := Ls[3];
  
  DB := Matrix(N+1,N+1);
  DC := Matrix(N+1,N+1);
  DB := []; DC := []; DQ2 :=[]; DQ3:= []; DR3 := [];
  for i from 1 to N+1 do
    DBI := []; DCI := [];
    for j from 1 to N+1 do
      DBI := [op(DBI),map(diff,Lp[i][2],Y[j])];
      DCI := [op(DCI),map(diff,Lp[i][3],Y[j])];
    od:
    DB := [op(DB),DBI];
    DC := [op(DC),DCI];
    DQ2 := [op(DQ2),map(diff,Ps[2],Y[i])];
    DQ3 := [op(DQ3),map(diff,Ps[4],Y[i])];
    DR3 := [op(DR3),map(diff,Ps[5],Y[i])];
  od:
  return([DB,DC,DQ2,DQ3,DR3]);
end proc:

mkdmat2:=proc(TPs,TLp,Ls,GRAD,VT)
  local i,j;
  local N,X,Y;
  local DB,DC,DBI,DCI,DQ2,DQ3,DR3;
  local H, G;
  
  N := Ls[1];
  X := Ls[2];
  Y := Ls[3];

  G := GRAD[(N+1)*(N+2)/2+1..nops(Ls[8])/2-1];
  
  DB := Matrix(N+1,N+1);
  DC := Matrix(N+1,N+1);
  DB := []; DC := []; DQ2 :=[]; DQ3:= []; DR3 := [];
  for i from 1 to N+1 do
    DBI := []; DCI := [];
    for j from 1 to N+1 do
      DBI := [op(DBI),map(diff,TLp[i][2],VT)/G[j]];
      DCI := [op(DCI),map(diff,TLp[i][3],VT)/G[j]];
    od:
    DB := [op(DB),DBI];
    DC := [op(DC),DCI];
    DQ2 := [op(DQ2),map(diff,TPs[2],VT)/G[i]];
    DQ3 := [op(DQ3),map(diff,TPs[4],VT)/G[i]];
    DR3 := [op(DR3),map(diff,TPs[5],VT)/G[i]];
  od:
  return([DB,DC,DQ2,DQ3,DR3]);
end proc:

pfaffian:=proc(F,V,VVal,IND,CGS,Sign)
  local i, j;
  local N, M;
  local Ps, Lp, DPs, Dmat, DB, DC, HH, H2, H3, HI, HJ, HIJ, AII;
  local P2I, P3I;
  local PF, SF, GS;

  GS := CGS;
#print(GS);

  Ps := F[1];
  Lp := F[2];
  DPs := F[3];
  N := RowDimension(Ps[3])-1;
  DB := DPs[1];
  DC := DPs[2];
  HH := [];

  P2I := MatrixInverse(Ps[1]);
  P3I := MatrixInverse(Ps[3]);

  if ( nops(IND) = 1) then # y ˴ؤ Pfaffian
    i := IND[1];
    AII := MatrixInverse(Lp[i][1]);
#    H2 := -P2I.Ps[2];
#    H3 := -P3I.(Ps[4].H2+P2I.Ps[5]);
#    PF := MatrixInverse(Lp[i][1]).(P2I.P3I.Lp[i][2]+P3I.Lp[i][3].H2+Ls[i][4].H3);
     PF := AII.(Lp[i][2]-Lp[i][3].P2I.Ps[2]+Lp[i][4].P3I.(Ps[4].P2I.Ps[2]-Ps[5]));
  else  # x ˴ؤ Pfaffian
    i := IND[1]; j := IND[2];
    AII := MatrixInverse(Lp[i][1]);
    HI := AII.(Lp[i][2]-Lp[i][3].P2I.Ps[2]+Lp[i][4].P3I.(Ps[4].P2I.Ps[2]-Ps[5]));
    HJ := MatrixInverse(Lp[j][1]).(Lp[j][2]-Lp[j][3].P2I.Ps[2]+Lp[j][4].P3I.(Ps[4].P2I.Ps[2]-Ps[5]));
    HIJ := AII.( DPs[1][i][j]-DPs[2][i][j].P2I.Ps[2]-Lp[i][3].P2I.DPs[3][j]+Lp[i][4].P3I.(DPs[4][j].P2I.Ps[2]+Ps[4].P2I.DPs[3][j]-DPs[5][j]) );
    PF := HIJ + HI.HJ;
  end if;
  
  M := RowDimension(PF);
  SF := Matrix(M,M,shape=identity);
  if ( Sign < 0 ) then GS := subs(V=-V,GS); end if;
  SF := subs(V=VVal,diff(GS,V))*SF;
  return(Sign*(PF+SF));
end proc:


exist:=proc(Name)
  local E, cmd;
  if ( Name = "" ) then return(0); end if;
  cmd := cat("test -e ",Name,"; echo $?");
  E := ssystem(cmd);
  if ( substring(E[2],1) = "0" ) then return(1); else return(0); end if;
end proc:

fb_int:=proc(N,Opt,Val)
  local E, B, run, i, Name, Fp, asirName;
  asirName := "tmp-fbmd-make.rr";
  Name := cat("fb",N,"d");
  run := cat("./tmp-",Name," --",Opt);
  if ( exist(cat("tmp-",Name)) = 0 ) then
    printf("compiling...\n");
    Fp := fopen(asirName,WRITE);
    fprintf(Fp,"load(\"fbmd.rr\");\n");
    fprintf(Fp,"get_fbmd(%d);\n",N+1);
    fprintf(Fp,"end$\n");
    fclose(Fp);
    E := ssystem(cat("asir < ",asirName));
    if ( E[1] <> 0 ) then error(""); end if;
  end if;

  for i from 1 to nops(Val) do
    run := cat(run,sprintf(" %f",Val[i]));
  od;
  printf(cat(run,"\n"));
  E := ssystem(run);
  if ( E[1] <> 0 ) then error(""); end if;
  printf(E[2]);
  B := read_res(Name);
  return(B);
end proc:

fb_int_tkoyama:=proc(N,Opt,Val)
  local E, B, run, i, Name, Fp, asirName, T;
  Name := "tkoyama-initial";
  run := cat("./",Name," --",Opt);
  if ( exist(cat(Name)) = 0 ) then
    printf("compiling...\n");
    E := ssystem(cat("gcc ",Name,".c -DSAMPLE=0 -lm -lgsl -lblas -o",Name));
    if ( E[1] <> 0 ) then error(""); end if;
  end if;

  for i from 1 to nops(Val) do
    run := cat(run,sprintf(" %f",Val[i]));
  od;
  printf(cat(run,"\n"));
  T := time[real]();
  E := ssystem(run);
  printf("time: %a\n",time[real]()-T);
  if ( E[1] <> 0 ) then error(""); end if;
  printf(E[2]);
  B := read_res(cat("fb",N,"d"));
  return(B);
end proc:

read_res:=proc(Name)
  local Fp, Line, IV;
  Fp := fopen(cat("tmp-out-",Name,".txt"),READ,TEXT);
  IV := [];
  Line := readline(Fp);
  while ( Line <> 0 ) do
    IV := [op(IV), parse(Line)];
    Line := readline(Fp);
  od;
  fclose(Fp);
  return(IV);
end proc:

set_var:=proc(N)
  local X,T,i,j,VX,VY;
  X := Matrix(N+1,N+1);
  T := [];
  VX := [];
  VY := [];
  for i from 1 to N+1 do
    T := [op(T),cat(t,i)];
    VY := [op(VY),cat(y,i)];
    for j from 1 to N+1 do
      if ( i <= j ) then
        X[i,j] := cat(x,i,j);
        VX := [op(VX),X[i,j]];
      else
        X[i,j] := cat(x,j,i);
      end if;
      if ( i <> j ) then X[i,j]:=X[i,j]/2; end if;
    od;
  od;
return(X,T,VX,VY);
end proc:

fb_integrand:=proc(N)
  local X,T,VX,VY,F;
  X,T,VX,VY := set_var(N);
  F := Vector[row](T).X.Vector(T)+Vector[row](T).Vector(VY);
  return(simplify(F),T);  #(simplify(F),X,T,VX,VY);
end proc:

polar:=proc(T,P)
  local N, Var, i, PC, M, J, PV, D;
  N := nops(T);
  Var := P[2];
  i := 1;
  M := P[1];
  PC := [T[i]=P[1]*cos(cat(P[2],i))];
  for i from 2 to N-1 do
    M := M*sin(cat(P[2],i-1));
    PC := [op(PC),T[i]=M*cos(cat(P[2],i))];
  od;
  PC := [op(PC),T[i]=M*sin(cat(P[2],i-1))];

  J := P[1]^(N-1);
  PV := [];
  D := [];
  for i from 1 to N-2 do
    J := J*sin(cat(P[2],i))^(N-i-1);
    PV := [op(PV),cat(P[2],i)];
    D := [op(D),[0,Pi]];
  od;
  PV := [op(PV),cat(P[2],i)];
  D := [op(D),[0,2*Pi]];
  return(PC,J,PV,D);
end proc:

read_point_data:=proc(Name)
  local Fp;
  local A, TL;
  option num;

  Fp := fopen(Name,READ,TEXT);

  A := readline(Fp); #1
  A := readline(Fp);
  TL := [];
  while ( A <> "];" ) do
    TL := [op(TL),parse(A)];
    A := readline(Fp);
  end do; 
  fclose(Fp);
  return(TL);
end proc:

mk_s:=proc(TL)
  local N, M;
  local i, j, k;
  local SM, SV, L;

  N := nops(TL);
  M := nops(TL[1]);
  SM := Matrix(M,M);
  SV := Vector(M);
  for k from 1 to N do
    L := TL[k];
    for i from 1 to M do
      SV[i] := SV[i] + L[i];
      for j from i to M do
        SM[i,j] := SM[i,j] + L[i]*L[j];
      od;
    od;
  od;
  return(SM/N,SV/N);
end proc:

x2s:=proc(X)
  return(cat(s,substring(X,2..)));
end proc:

slist:=proc(SX,SY)
  local i, j, N, SL;

  SL := [];
  N := op(1,SY);
  for i from 1 to N do
    for j from i to N do
      SL := [op(SL),SX[i,j]];
    od;
  od;
  return([op(SL),op(convert(SY,`list`))]);
end proc:
